<?php
#-------------------------------------------------------------------------------
#	$Id: longtrack.php,v 1.2 2023/01/11 23:24:11 wolf Exp $
#-------------------------------------------------------------------------------
#
#	Dynamically "generalizes" a track using a greedy algorithm
#
#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
#
#	* * * * * HIER ANPASSEN * * * * *
#
#-------------------------------------------------------------------------------

$TRACK_DIRECTORY = ".";

#-------------------------------------------------------------------------------
#
#	Fehlerbehandlung
#
#-------------------------------------------------------------------------------

function Fail( $message ){

	header( "HTTP/1.0 400 Bad Request" );
	header( "Content-Type: text/plain; charset=UTF-8" );

	echo "Error: $message\n";
	exit;
}

#-------------------------------------------------------------------------------
#
#	Format: gpx
#
#-------------------------------------------------------------------------------

function httpHeader( ){

	header( "Content-Type: application/gpx+xml" );
}

function preTrack( $name ){

	echo <<<EOT
<?xml version="1.0" encoding="UTF-8"?>
<gpx version="1.0" creator="longtrack.php" xmlns="http://www.topografix.com/GPX/1/0">
<trk>
<name>$name</name>

EOT;
}

function preSegment( ){

	echo "<trkseg>\n";
}

function point( $lon, $lat ){

	echo "<trkpt lon=\"$lon\" lat=\"$lat\"></trkpt>\n";
}

function betweenPoints(){}

function postSegment( ){

	echo "</trkseg>\n";
}

function betweenSegments(){}

function postTrack( ){

	echo "</trk>\n</gpx>\n";
}

#-------------------------------------------------------------------------------
#
#	parameter bbox (required)
#
#-------------------------------------------------------------------------------

if( !preg_match( '/^(-?\d+\.?\d*),(-?\d+\.?\d*),(-?\d+\.?\d*),(-?\d+\.?\d*)$/',
		$_GET[ 'bbox' ], $match ) ){

	Fail( "Parameter 'bbox' missing or invalid" );
}

$MinLon = floatval( $match[ 1 ] );
$MinLat = floatval( $match[ 2 ] );
$MaxLon = floatval( $match[ 3 ] );
$MaxLat = floatval( $match[ 4 ] );

#-------------------------------------------------------------------------------
#
#	parameter mindist (min lon distance in degrees )
#
#-------------------------------------------------------------------------------

if( !preg_match( '/^(\d+\.?\d*)$/', $_GET[ 'mindist' ], $match ) ){

	Fail( "Parameter 'bbox' missing or invalid" );
}

$MinLonDist = floatval( $match[ 1 ] );

#-------------------------------------------------------------------------------
#
#	A liitle bit of math ...
#
#-------------------------------------------------------------------------------

$MeanLat       = ( $MinLat + $MaxLat ) / 2;
$MercatorScale = cos( deg2rad( $MeanLat ));
$MinLatDist    = $MinLonDist * $MercatorScale;

#-------------------------------------------------------------------------------
#
#	parameter track (required)
#
#-------------------------------------------------------------------------------

if( !preg_match( '/^(\w+(?:-\w+)*)$/', $_GET[ 'track' ], $match )){

	Fail( "Parameter 'track' missing or invalid." );
}

$TrackName = $match[ 1 ];

#-------------------------------------------------------------------------------
#	Track-File
#-------------------------------------------------------------------------------

$TrackFileName = "$TRACK_DIRECTORY/$TrackName.txt";

#-------------------------------------------------------------------------------
#	open Track file
#-------------------------------------------------------------------------------

$TrackFile = @fopen( $TrackFileName, "r" );

if( !$TrackFile ){

	header( "HTTP/1.0 404 Not Found" );
	header( "Content-Type: text/plain; charset=UTF-8" );

	echo "Cannot open '$TrackFileName'.\n";	# verraet das Verzeichnis
	#echo "Cannot open '$TrackName'.\n";	# verraet das Verzeichnis nicht
	exit;
}

#-------------------------------------------------------------------------------
#
#	HTTP header and Track header
#
#-------------------------------------------------------------------------------

httpHeader();
preTrack( $TrackName );

#-------------------------------------------------------------------------------
#
#	Init algorithm
#
#-------------------------------------------------------------------------------

$lastLon = 1e9;
$lastLat = 1e9;

$inSegment = 0;
$outLon;
$outLat;

$nSegments = 0;
$nPoints   = 0;

#-------------------------------------------------------------------------------
#
#	process points
#
#-------------------------------------------------------------------------------

while( $line = fgets( $TrackFile ) ){

	#-----------------------------------------------------------------------
	#	gap
	#-----------------------------------------------------------------------

	if( !preg_match( '!^(-?\d+\.?\d*)\s+(-?\d+\.?\d*)\s*$!',
							$line, $match ) ){

		if( $inSegment ){ postSegment(); $inSegment = 0; }
		$lastLon = 1e9;
		$lastLat = 1e9;
		unset( $outLon );
		unset( $outLat );
		continue;
	}

	$lon = $match[ 1 ];
	$lat = $match[ 2 ];

	#-----------------------------------------------------------------------
	#	generalize by simply dropping points (greedy algorithm)
	#-----------------------------------------------------------------------

	if(	abs( $lon - $lastLon ) < $MinLonDist &&
		abs( $lat - $lastLat ) < $MinLatDist ){ continue; }

	$lastLon = $lon;
	$lastLat = $lat;

	#-----------------------------------------------------------------------
	#	This was the very generalization algorithm.
	#-----------------------------------------------------------------------
	#	Now discard segments outside our bbox.
	#-----------------------------------------------------------------------

	#-----------------------------------------------------------------------
	#	handle point inside bbox
	#-----------------------------------------------------------------------

	if(	$lon >= $MinLon && $lon <= $MaxLon &&
		$lat >= $MinLat && $lat <= $MaxLat) {

		#---------------------------------------------------------------
		#	start segment if needed
		#---------------------------------------------------------------

		if( !$inSegment ){

			$inSegment = 1;
			$nPoints   = 0;

			#-------------------------------------------------------
			#	segment joiner if needed
			#-------------------------------------------------------

			if( $nSegments++ ){ betweenSegments(); }

			#-------------------------------------------------------
			#	points prefix
			#-------------------------------------------------------

			preSegment();

			#-------------------------------------------------------
			#	line segment crossing from outside to inside?
			#-------------------------------------------------------

			if( isset( $outLon ) && isset( $outLat ) ){

				point( $outLon, $outLat );
				unset( $outLon );
				unset( $outLat );
				++$nPoints;
			}
		}

		#---------------------------------------------------------------
		#	point joiner if needed
		#---------------------------------------------------------------

		if( $nPoints++ ){ betweenPoints(); }

		#---------------------------------------------------------------
		#	point inside bbox
		#---------------------------------------------------------------

		point( $lon, $lat);

	#-----------------------------------------------------------------------
	#	handle point outside bbox
	#-----------------------------------------------------------------------

	} else {

		$outLon = $lon;
		$outLat = $lat;

		#---------------------------------------------------------------
		#	line segment crossing from inside to outside?
		#---------------------------------------------------------------

		if( $inSegment ){

			#-------------------------------------------------------
			#	point joiner if needed
			#-------------------------------------------------------

			if( $nPoints++ ){ betweenPoints(); }

			#-------------------------------------------------------
			#	point outside bbox
			#-------------------------------------------------------

			point( $outLon, $outLat );

			#-------------------------------------------------------
			#	points suffix
			#-------------------------------------------------------

			postSegment();
			$inSegment = 0;
		}
	}
}

#-------------------------------------------------------------------------------
#	points suffix if needed
#-------------------------------------------------------------------------------

if( $inSegment ){ postSegment(); }

#-------------------------------------------------------------------------------
#	segments suffix
#-------------------------------------------------------------------------------

postTrack();

exit( 0 );

#-------------------------------------------------------------------------------
#	$Id: longtrack.php,v 1.2 2023/01/11 23:24:11 wolf Exp $
#-------------------------------------------------------------------------------
